<?php
/**
 * Plugin Helper File
 *
 * @package     Cache Cleaner
 * @version     1.6.0
 *
 * @author      Peter van Westen <peter@nonumber.nl>
 * @link        http://www.nonumber.nl
 * @copyright   Copyright © 2010 NoNumber! All Rights Reserved
 * @license     http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
 */

// No direct access
defined( '_JEXEC' ) or die( 'Restricted access' );

/**
* Plugin that cleans cache
*/
class plgSystemCacheCleanerHelper
{
	function plgSystemCacheCleanerHelper( &$params, $type = 'clean', $show_msg = 1, $show_size = 0 )
	{
		jimport('joomla.filesystem.folder');
		jimport('joomla.filesystem.file');

		$final_state = 1;

		$size = 0;

		if ( $show_size ) {
			$size += $this->getFolderSize( JPATH_SITE.DS.'cache' );
			$size += $this->getFolderSize( JPATH_ADMINISTRATOR.DS.'cache' );
		}

		$cache =& JFactory::getCache();

		// remove all folders in cache folder
		$cache->clean();
		$cache->gc();

		// remove all remaining folders and files in cache folder
		$paths = array( JPATH_SITE, JPATH_ADMINISTRATOR );
		foreach ( $paths as $path ) {
			$path .= DS.'cache';
			list( $final_state, $s ) = $this->emptyFolder( $path );
		}

		if ( $show_size ) {
			$size -= $this->getFolderSize( JPATH_SITE.DS.'cache' );
			$size -= $this->getFolderSize( JPATH_ADMINISTRATOR.DS.'cache' );
		}

		// Empty cmslib cache
		if ( $params->clean_cmslib ) {
			$path = JPATH_SITE.DS.'components'.DS.'libraries'.DS.'cmslib'.DS.'cache';
			list( $final_state, $s ) = $this->emptyFolder( $path, $show_size );
			if ( $show_size ) {
				$size += $s;
			}
		}


		// Empty JRE cache db table
		if ( $params->clean_jre ) {
			$db =& JFactory::getDBO();
			$db->setQuery( 'show tables like \''.$db->_table_prefix.'jrecache_repository\'' );
			$exists = $db->loadResult();
			if ( $exists ) {
				$db->setQuery( 'TRUNCATE TABLE `#__jrecache_repository`' );
				$db->query();
			}
		}

		// Folders
		if (	$type == 'clean'
			||	( $type == 'interval' && $params->auto_interval_folders )
			||	( $type == 'save' && $params->auto_save_folders )
		) {
			// Empty tmp folder
			if ( $params->clean_tmp ) {
				$path = JPATH_SITE.DS.'tmp';
				list( $final_state, $s ) = $this->emptyFolder( $path, $show_size );
				if ( $show_size ) {
					$size += $s;
				}
			}
			// Empty custom folder
			if ( $params->clean_folders ) {
				$folders = explode( '\n', $params->clean_folders );
				foreach( $folders as $folder ) {
					if ( trim( $folder ) ) {
						$folder = str_replace( array( '\\', '/' ), DS, trim( $folder ) );
						$path = str_replace( DS.DS, DS, JPATH_SITE.DS.$folder );
						list( $final_state, $s ) = $this->emptyFolder( $path, $show_size );
						if ( $show_size ) {
							$size += $s;
						}
					}
				}
			}
		}

		// Write current time to text file
		$file = JPATH_PLUGINS.DS.'system'.DS.'cachecleaner'.DS.'lastclean.txt';
		JFile::write( $file, time() );

		// Load language for messaging
		$lang =& JFactory::getLanguage();
		$lang->load( 'mod_cachecleaner', JPATH_ADMINISTRATOR );
		$file = JPATH_ADMINISTRATOR.DS.'language'.DS.'en-GB'.DS.'en-GB.mod_cachecleaner.ini';
		$lang->_load( $file, 'mod_cachecleaner', 0 );

		$error = 0;
		if ( !$final_state ) {
			$msg = JText::_( 'CC_NOT_ALL_CACHE_COULD_BE_REMOVED' );
			$error = 1;
		} else {
			$msg = JText::_( 'CC_CACHE_CLEANED' );
		}

		if ( $show_size ) {
			if ( $size >= 1048576) {
				$size = ( round( $size / 1048576 * 100 ) / 100 ).'MB';
			} else {
				$size = ( round( $size / 1024 * 100 ) / 100 ).'KB';
			}
			$msg .= ' ('.$size.')';
		}

		if( JRequest::getInt( 'break' ) ) {
			echo ( $final_state ? '+' : '' ).$msg;
			exit;
		} else if ( $show_msg ) {
			$mainframe =& JFactory::getApplication();
			$mainframe->enqueueMessage( $msg, ( $error ? 'error' : 'message' ) );
		}
	}

	function emptyFolder( $path, $show_size = 0 )
	{
		jimport('joomla.filesystem.folder');
		jimport('joomla.filesystem.file');

		$succes = 1;
		$size = 0;

		if ( JFolder::exists( $path ) ) {
			if ( $show_size ) {
				$size = $this->getFolderSize( $path );
			}
			// remove folders
			$folders = JFolder::folders( $path );
			foreach ( $folders as $folder ) {
				$succes = JFolder::delete( $path.DS.$folder );
			}
			// remove files
			$files = JFolder::files( $path );
			foreach ( $files as $file ) {
				if ( $file != 'index.html' ) {
					$succes = JFile::delete( $path.DS.$file );
				}
			}
			if ( $show_size ) {
				$size -= $this->getFolderSize( $path );
			}
		}

		return array( $succes, $size );
	}

	function getFolderSize( $path )
	{
		jimport('joomla.filesystem.file');

		if( JFile::exists( $path ) ) {
			return filesize( $path );
		}

		jimport('joomla.filesystem.folder');
		if( !JFolder::exists( $path ) ) {
			return 0;
		}

		$size = 0;
		foreach( JFolder::files( $path ) as $file ) {
			$size += filesize( $path.DS.$file );
		}
		foreach( JFolder::folders( $path ) as $folder ) {
			$size += $this->getFolderSize( $path.DS.$folder );
		}

		return $size;
	}
}